@@ -117,6 +117,9 @@ finalize (CamelExchangeStore *exch)
 		camel_object_unref (CAMEL_OBJECT (exch->stub));
 		exch->stub = NULL;
 	}
+	
+	g_free (exch->trash_name);
+	
 	if (exch->folders_lock)
 		g_mutex_free (exch->folders_lock);
 }
@@ -219,6 +222,7 @@ exchange_path_to_physical (const char *prefix, const char *vpath)
 
 	return ppath;
 }
+
 static void
 construct (CamelService *service, CamelSession *session,
 	   CamelProvider *provider, CamelURL *url, CamelException *ex)
@@ -237,10 +241,13 @@ construct (CamelService *service, CamelSession *session,
 			*p = '\0';
 	}
 
-	exch->storage_path = camel_session_get_storage_path (session, service, ex);
-
+	if (!(exch->storage_path = camel_session_get_storage_path (session, service, ex)))
+		return;
+	
 	if (camel_url_get_param (url, "filter_junk"))
 		CAMEL_STORE (service)->flags |= CAMEL_STORE_VJUNK;
+	
+	exch->stub = NULL;
 }
 
 extern CamelServiceAuthType camel_exchange_password_authtype;
@@ -271,12 +278,12 @@ get_name (CamelService *service, gboolean brief)
 static gboolean
 exchange_connect (CamelService *service, CamelException *ex)
 {
-	CamelExchangeStore *store = CAMEL_EXCHANGE_STORE (service);
+	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
 	char *real_user, *socket_path;
-	
-	if (!store->storage_path)
-		return FALSE;
-	
+
+	if (exch->stub)
+		return TRUE;
+
 	real_user = strpbrk (service->url->user, "\\/");
 	if (real_user)
 		real_user++;
@@ -287,48 +294,37 @@ exchange_connect (CamelService *service, CamelException *ex)
 				       real_user, service->url->host);
 	e_filename_make_safe (strchr (socket_path + 5, '/') + 1);
 	
-	store->stub = camel_stub_new (socket_path, _("Evolution Exchange backend process"), ex);
+	exch->stub = camel_stub_new (socket_path, _("Evolution Exchange backend process"), ex);
 	g_free (socket_path);
-	if (!store->stub)
+	if (!exch->stub)
 		return FALSE;
-
+	
 	/* Initialize the stub connection */
-	if (!camel_stub_send (store->stub, NULL, CAMEL_STUB_CMD_CONNECT,
+	if (!camel_stub_send (exch->stub, NULL, CAMEL_STUB_CMD_CONNECT,
 			      CAMEL_STUB_ARG_RETURN,
 			      CAMEL_STUB_ARG_END)) {
 		/* The user cancelled the connection attempt. */
 		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
 				     "Cancelled");
-		camel_object_unref (CAMEL_OBJECT (store->stub));
-		store->stub = NULL;
+		camel_object_unref (exch->stub);
+		exch->stub = NULL;
 		return FALSE;
 	}
-
-	camel_object_hook_event (CAMEL_OBJECT (store->stub), "notification",
-				 stub_notification, store);
-
+	
+	camel_object_hook_event (CAMEL_OBJECT (exch->stub), "notification",
+				 stub_notification, exch);
 	return TRUE;
 }
 
 static gboolean
 exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
-	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
-
-	if (exch->stub) {
-		camel_object_unref (CAMEL_OBJECT (exch->stub));
-		exch->stub = NULL;
-	}
-
-	g_free (exch->trash_name);
-	exch->trash_name = NULL;
-
 	return TRUE;
 }
 
-#define RETURN_VAL_IF_NOT_CONNECTED(service, ex, val) \
-	if (!camel_service_connect ((CamelService *)service, ex)) \
-		return val;
+#define RETURN_VAL_IF_NOT_CONNECTED(service, ex, val)\
+	if (!exchange_connect((CamelService *)service, ex))\
+		return val; 
 
 static CamelFolder *
 exchange_get_folder (CamelStore *store, const char *folder_name,
@@ -338,8 +334,8 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 	CamelFolder *folder;
 	char *folder_dir;
 	
-	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
-	
+	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
+
 	folder_dir = exchange_path_to_physical (exch->storage_path, folder_name);
 	
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
@@ -389,8 +385,8 @@ static CamelFolder *
 get_trash (CamelStore *store, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
-
-	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
+	
+	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
 
 	if (!exch->trash_name) {
 		if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_TRASH_NAME,
@@ -478,11 +474,13 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 		return NULL;
 	}
 #endif	
-	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
-
+	
 	/* If the backend crashed, don't keep returning an error
 	 * each time auto-send/recv runs.
 	 */
+
+	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
+
 	if (camel_stub_marshal_eof (exch->stub->cmd))
 		return NULL;
 
